@@ -12,29 +12,35 @@ sub acceptance {
   my ($class, $schema_class, %acceptance_params) = @_;
 
   Test::More::plan(skip_all => 'cpanm Test::JSON::Schema::Acceptance')
-    unless eval 'use Test::JSON::Schema::Acceptance 1.000;1';
-  Test::More::plan(skip_all => 'cpanm Test2::Tools::Compare') unless eval 'use Test2::Tools::Compare 0.0001;1';
+    unless eval 'use Test::JSON::Schema::Acceptance 1.000 ();1';
+  Test::More::plan(skip_all => 'cpanm Test2::Tools::Compare') unless eval 'use Test2::Tools::Compare 0.0001 ();1';
   Test::More::plan(skip_all => $@)                            unless eval "require $schema_class;1";
 
-  $acceptance_params{todo_tests}
-    = [map { +{file => $_->[0], group_description => $_->[1], test_description => $_->[2]}; }
-      @{$acceptance_params{todo_tests}}]
+  my $test = sub { +{file => $_[0], group_description => $_[1], test_description => $_[2]} };
+
+  $acceptance_params{todo_tests} = [map { $test->(@$_) } @{$acceptance_params{todo_tests}}]
     if $acceptance_params{todo_tests};
 
   my $specification = $schema_class =~ m!::(\w+)$! ? lc $1 : 'unknown';
+  $specification = 'draft2019-09' if $specification eq 'draft201909';
   Test::JSON::Schema::Acceptance->new(specification => $specification)->acceptance(
+    tests => $test->(split '/', $ENV{TEST_ACCEPTANCE} || ''),
     %acceptance_params,
     validate_data => sub {
       my ($schema_p, $data_p) = map { Mojo::JSON::Pointer->new(shift @_) } qw(schema data);
-      my ($schema,   $data)   = map { clone($_->data) } $schema_p, $data_p;
-      my $valid = $schema_class->new($schema)->validate($data) ? 0 : 1;
+      my ($schema_d, $data_d) = map { clone($_->data) } $schema_p, $data_p;
+
+      my $schema = $schema_class->new($schema_d);
+      return 0 if @{$schema->errors};
+
+      my @errors = $schema->validate($data_d);
 
       # Doing internal tests on mutation, since I think Test::JSON::Schema::Acceptance is a bit too strict
-      Test2::Tools::Compare::is(encode_json($data),   encode_json($data_p->data),   'data structure is the same');
-      Test2::Tools::Compare::is(encode_json($schema), encode_json($schema_p->data), 'schema structure is the same')
+      Test2::Tools::Compare::is(encode_json($data_d),   encode_json($data_p->data),   'data structure is the same');
+      Test2::Tools::Compare::is(encode_json($schema_d), encode_json($schema_p->data), 'schema structure is the same')
         unless _acceptance_schema_contains_invalid_ref($schema_p);
 
-      return $valid;
+      return @errors ? 0 : 1;
     },
   );
 }
